import processing.serial.*;

int numNurse = 13;
int numStatus = 4;
int current_nurse = 10;
int current_status = 0;

PImage bg;
PImage[] nurse_images = new PImage[numNurse];
PImage[] status_images = new PImage[numStatus];

int ACCEL_LAYING[] = {44,50,63};
int ACCEL_SIDE[] = {44,50,63};
int ACCEL_ERROR = 5;
int ACCEL_KILL = 20;

Game game;

void setup()
{
  game = new Game();
  game.frame_rate = 30;

  size(796, 599);
  frameRate(game.frame_rate);
  //      ---Nurse Image Array---
  nurse_images[0]  = loadImage("air.jpg");
  nurse_images[1]  = loadImage("compression.jpg"); 
  nurse_images[2]  = loadImage("defibrillator.jpg");
  nurse_images[3]  = loadImage("intubate.jpg"); 
  nurse_images[4]  = loadImage("bad_idea.jpg");
  nurse_images[5]  = loadImage("compliment.jpg"); 
  nurse_images[6]  = loadImage("fool.jpg");
  nurse_images[7]  = loadImage("good_call.jpg"); 
  nurse_images[8]  = loadImage("great_job.jpg");
  nurse_images[9]  = loadImage("lost.jpg"); 
  nurse_images[10] = loadImage("next_patient.jpg");
  nurse_images[11] = loadImage("side_first.jpg"); 
  nurse_images[12] = loadImage("something_else.jpg"); 

  //      ---Status Image Array---
  status_images[0]  = loadImage("status_normal.jpg");
  status_images[1]  = loadImage("status_bad.jpg");
  status_images[2]  = loadImage("status_worse.jpg");
  status_images[3]  = loadImage("status_dead.jpg");

  //      ---Load Background---
  bg = loadImage("background2.jpg");

  //the following may need to be changed to a specific serial port
  println(Serial.list());
  game.serial = new Serial(this,Serial.list()[0],9600);
  game.current_state = new Start(game);  
  game.begin();
}

void draw()
{
  game.update();

  background(bg);
  image(nurse_images[game.current_state.current_nurse],38, 28);
  image(status_images[game.current_state.current_status],475, 82);
}

class Game
{
  int frame_rate;
  //inputs over serial
  byte accel[] = {0,0,0};
  byte chest_switch;
  byte mouth_switch;
  byte paddle_switch;
  byte reset_switch;

  int total_flatlines;
  int error_movement;
  int kill_movement;
  byte buffer[] = new byte[8];

  Serial serial;
  BearState current_state;
  BearState next_state;

  Game()
  {
    this.total_flatlines = 0;
    this.error_movement = 20;
    this.kill_movement = 100;
  }

  void begin()
  {
    this.current_state.startState();
  }

  void update()
  {
    if (this.current_state.flatline)
    {
      this.serial.write("f");
    }
    else if (this.current_state.convulse)
    {
      this.serial.write("c");
    }
    else if (this.current_state.shocked)
    {
      this.serial.write("d");
    }
    else if (this.current_state.slow_beat)
    {
      this.serial.write("s");
    }
    else
    {
      this.serial.write("n");
    }
    //n = normal
    //e = elevated
    //c = convulse
    //d = defib
    //s = shallow
    //f = flatline
    this.serial.readBytesUntil(10,this.buffer);

    this.accel[0] = this.buffer[0];
    this.accel[1] = this.buffer[1];
    this.accel[2] = this.buffer[2];
    this.chest_switch = this.buffer[3];
    this.mouth_switch = this.buffer[4];
    this.paddle_switch = this.buffer[5];
    this.reset_switch = this.buffer[6];
    
    print("Input: " + this.accel[0] + " " + this.accel[1] + " " + this.accel[2] + " ");
    println(this.chest_switch + " " + this.mouth_switch + " " + this.paddle_switch + " " + this.reset_switch);

    this.current_state.checkInputs();
    this.next_state = this.current_state.changeState();
    if (this.next_state != null)
    {
      this.current_state = this.next_state;
      this.current_state.startState();
    }
    this.current_state.loopState();
    //print(this.current_state.time);
    //print(" ");
    //println(this.current_state.current_status);
  }
}

class BearState
{
  Game game;
  int time;
  int current_nurse = 5;
  int current_status = 3;

  //inputs
  int comps = 0;
  boolean comping = false;
  int pumps = 0;
  boolean pumping = false;
  int paddles = 0;
  boolean paddling = false;
  
  boolean laying = false;
  boolean side = false;

  //outputs
  boolean flatline = false;
  boolean convulse = false;
  boolean normal_beat = false;
  boolean slow_beat = false;
  boolean shocked = false;

  BearState(Game game)
  {
    this.game = game;
  }

  void startState()
  {
    this.time = 0;
  }

  void loopState()
  {
    this.time += 1;
  }

  BearState changeState()
  {
    return null;
  }

  void checkInputs()
  {
    if (this.game.chest_switch == 1 && !this.comping)
    {
      this.comping = true;
      this.comps +=1;
    }
    if (this.game.chest_switch == 0 && this.comping)
    {
      this.comping = false;
    }

    if (this.game.mouth_switch == 1 && !this.pumping)
    {
      this.pumping = true;
      this.pumps +=1;
    }
    if (this.game.mouth_switch == 0 && this.pumping)
    {
      this.pumping = false;
    }

    if (this.game.chest_switch == 1 && !this.paddling)
    {
      this.paddling = true;
      this.paddles +=1;
    }
    if (this.game.chest_switch == 0 && this.paddling)
    {
      this.paddling = false;
    }
    
    int i;
    
    this.laying = true;
    for(i = 0; i < 3; i++)
    {
      if (abs(ACCEL_LAYING[i]-this.game.accel[i]) > ACCEL_ERROR ) this.laying = false;
    }

    this.side = true;
    for(i = 0; i < 3; i++)
    {
      if (abs(ACCEL_SIDE[i]-this.game.accel[i]) > ACCEL_ERROR ) this.side = false;
    }
  }

  int framesToSeconds(int frames)
  {
    return frames/this.game.frame_rate;
  }
}

class Start extends BearState
{
  Start(Game game)
  {
    super(game);
  }

  void startState()
  {
    super.startState();
    this.game.total_flatlines = 0;
    this.current_nurse = 10;
    this.current_status = 0;
  }

  void loopState()
  {
    super.loopState();
  }

  BearState changeState()
  {
    int change_time = 5;
    randomSeed(millis());
    float i = random(3);
    if (i <= 1.0 & this.framesToSeconds(this.time) > change_time)
    {
      return new ConvulseCardio(this.game);
    }
    else if (i <= 2.0 & this.framesToSeconds(this.time) > change_time)
    {
      return new ConvulseFlatline(this.game);
    }
    else if (i <= 3.0 & this.framesToSeconds(this.time) > change_time)
    {
      return new ConvulseGI(this.game);
    }
    else
    {
      return null;
    }
  }
}

class ConvulseCardio extends BearState
{
  ConvulseCardio(Game game)
  {
    super(game);
  }

  void startState()
  {
    super.startState();
    this.current_nurse = 1;
    this.current_status = 1;
    this.convulse = true;
  }

  void loopState()
  {
    super.loopState();
  }

  BearState changeState()
  {
    int change_time = 20;
    randomSeed(millis());
    float i = random(3);

    if (this.comps >= 6 && this.framesToSeconds(this.time) < change_time && this.laying)
    {
      return new Stable(this.game);
    }
    else if (this.framesToSeconds(this.time) > change_time)
    {
      return new Death(this.game);
    }
    else
    {
      return null;
    }
  }
}

class ConvulseFlatline extends BearState
{
  ConvulseFlatline(Game game)
  {
    super(game);
  }

  void startState()
  {
    super.startState();
    this.current_nurse = 12;
    this.current_status = 3;
    this.convulse = true;
  }

  void loopState()
  {
    super.loopState();
  }

  BearState changeState()
  {
    int change_time = 10;

    if (this.framesToSeconds(this.time) > change_time)
    {
      return new AnyTouch(this.game);
    }
    else
    {
      return null;
    }
  }
}

class ConvulseGI extends BearState
{
  ConvulseGI(Game game)
  {
    super(game);
  }

  void startState()
  {
    super.startState();
    this.current_nurse = 3;
    this.current_status = 2;
    this.convulse = true;
  }

  void loopState()
  {
    super.loopState();
  }

  BearState changeState()
  {
    int change_time = 5;

    if (this.framesToSeconds(this.time) < change_time)
    {
      return new ShallowVitals(this.game);
    }
    else
    {
      return null;
    }
  }
}

class Stable extends BearState
{
  Stable(Game game)
  {
    super(game);
  }

  void startState()
  {
    super.startState();
    this.current_nurse = 7;
    this.current_status = 0;
    this.normal_beat = true;
  }

  void loopState()
  {
    super.loopState();
  }

  BearState changeState()
  {
    int change_time = 10;

    if (this.framesToSeconds(this.time) < change_time && !this.laying)
    {
      return new NeedMoreComps(this.game);
    }
    else if (this.framesToSeconds(this.time) > change_time)
    {
      return new Well(this.game);
    }
    else
    {
      return null;
    }
  }
}

class NeedMoreComps extends BearState
{
  NeedMoreComps(Game game)
  {
    super(game);
  }

  void startState()
  {
    super.startState();
    this.current_nurse = 4;
    this.current_status = 2;
    this.convulse = true;
  }

  void loopState()
  {
    super.loopState();
  }

  BearState changeState()
  {
    int change_time = 10;

    if (this.comps >= 6 && this.framesToSeconds(this.time) < change_time && this.laying)
    {
      return new Stable(this.game);
    }
    else if ((this.comps < 6 || !this.laying) && this.framesToSeconds(this.time) > change_time && this.framesToSeconds(this.time) < change_time * 2)
    {
      return new NeedAir(this.game);
    }
    else if (this.framesToSeconds(this.time) > change_time * 2)
    {
      return new Death(this.game);
    }
    else
    {
      return null;
    }
  }
}

class NeedAir extends BearState
{
  NeedAir(Game game)
  {
    super(game);
  }

  void startState()
  {
    super.startState();
    this.current_nurse = 0;
    this.current_status = 2;
    this.convulse = true;
  }

  void loopState()
  {
    super.loopState();
  }

  BearState changeState()
  {
    int change_time = 20;

    if (this.pumps >= 6 && this.framesToSeconds(this.time) < change_time && this.laying)
    {
      return new Stable(this.game);
    }
    else if (this.framesToSeconds(this.time) > change_time)
    {
      return new Death(this.game);
    }
    else
    {
      return null;
    }
  }
}

class AnyTouch extends BearState
{
  AnyTouch(Game game)
  {
    super(game);
  }

  void startState()
  {
    super.startState();
    this.current_nurse = 6;
    this.current_status = 3;
    this.convulse = true;
  }

  void loopState()
  {
    super.loopState();
  }

  BearState changeState()
  {
    int change_time = 10;

    if ((this.comps > 0 || this.pumps > 0 || this.paddles > 0 || !this.laying) 
      && this.framesToSeconds(this.time) <= change_time)
    {
      return new Flatline(this.game);
    }
    else if (this.framesToSeconds(this.time) > change_time)
    {
      return new Death(this.game);
    }
    else
    {
      return null;
    }
  }
}

class Flatline extends BearState
{
  Flatline(Game game)
  {
    super(game);
  }

  void startState()
  {
    super.startState();
    this.current_nurse = 6;
    this.current_status = 3;
    this.flatline = true;
  }

  void loopState()
  {
    super.loopState();
  }

  BearState changeState()
  {
    int change_time = 20;

    if (this.game.total_flatlines < 3 && this.paddles > 0 && this.framesToSeconds(this.time) <= change_time)
    {
      return new Paddled(this.game);
    }
    else if (this.framesToSeconds(this.time) > change_time || this.game.total_flatlines > 3)
    {
      return new Death(this.game);
    }
    else
    {
      return null;
    }
  }
}

class Paddled extends BearState
{
  Paddled(Game game)
  {
    super(game);
  }

  void startState()
  {
    super.startState();
    this.current_nurse = 6;
    this.current_status = 3;
    this.flatline = true;
  }

  void loopState()
  {
    super.loopState();
  }

  BearState changeState()
  {
    int change_time = 1;
    randomSeed(millis());
    float i = random(3);
    if (i <= 1.0 & this.framesToSeconds(this.time) < change_time)
    {
      return new NeedAir(this.game);
    }
    else if (i <= 2.0 & this.framesToSeconds(this.time) < change_time)
    {
      return new Flatline(this.game);
    }
    else if (i <= 3.0 & this.framesToSeconds(this.time) < change_time)
    {
      return new Well(this.game);
    }
    else
    {
      return null;
    }
  }
}

class ShallowVitals extends BearState
{
  ShallowVitals(Game game)
  {
    super(game);
  }

  void startState()
  {
    super.startState();
    this.current_nurse = 3;
    this.current_status = 2;
    this.slow_beat = true;
  }

  void loopState()
  {
    super.loopState();
  }

  BearState changeState()
  {
    int change_time = 10;

    if (this.pumps > 0 && !this.side && this.framesToSeconds(this.time) < change_time)
    {
      return new TubeConvulse(this.game);
    }
    else if (this.side && this.framesToSeconds(this.time) < change_time)
    {
      return new ReadyToIntubate(this.game);
    }
    else if (this.framesToSeconds(this.time) > change_time)
    {
      return new Death(this.game);
    }
    else
    {
      return null;
    }
  }
}

class TubeConvulse extends BearState
{
  TubeConvulse(Game game)
  {
    super(game);
  }

  void startState()
  {
    super.startState();
    this.current_nurse = 11;
    this.current_status = 2;
    this.convulse = true;
  }

  void loopState()
  {
    super.loopState();
  }

  BearState changeState()
  {
    int change_time = 15;

    if (!this.pumping && this.framesToSeconds(this.time) < change_time)
    {
      return new ShallowVitals(this.game);
    }
    else if (this.framesToSeconds(this.time) >= change_time)
    {
      return new Death(this.game);
    }
    else
    {
      return null;
    }
  }
}

class ReadyToIntubate extends BearState
{
  ReadyToIntubate(Game game)
  {
    super(game);
  }

  void startState()
  {
    super.startState();
    this.current_nurse = 7;
    this.current_status = 1;
    this.convulse = true;
  }

  void loopState()
  {
    super.loopState();
  }

  BearState changeState()
  {
    int change_time = 15;

    if (this.pumping && this.framesToSeconds(this.time) < change_time)
    {
      return new Intubated(this.game);
    }
    else if (this.framesToSeconds(this.time) >= change_time)
    {
      return new Death(this.game);
    }
    else
    {
      return null;
    }
  }
}

class Intubated extends BearState
{
  Intubated(Game game)
  {
    super(game);
  }

  void startState()
  {
    super.startState();
    this.current_nurse = 5;
    this.current_status = 0;
    this.normal_beat = true;
  }

  void loopState()
  {
    super.loopState();
  }

  BearState changeState()
  {
    int change_time = 10;

    if (this.framesToSeconds(this.time) < change_time && !this.side)
    {
      return new TubeConvulse(this.game);
    }
    else if (this.framesToSeconds(this.time) > change_time)
    {
      return new Well(this.game);
    }
    else
    {
      return null;
    }
  }
}

class Death extends BearState
{
  Death(Game game)
  {
    super(game);
  }

  void startState()
  {
    super.startState();
    this.current_nurse = 9;
    this.current_status = 3;
    this.flatline = true;
  }

  void loopState()
  {
    super.loopState();
  }

  BearState changeState()
  {
    int change_time = 10;
    if (this.framesToSeconds(this.time) > change_time)
    {
      return new Start(this.game);
    } 
    else
    {
      return null;
    }
  }
}

class Well extends BearState
{
  Well(Game game)
  {
    super(game);
  }

  void startState()
  {
    super.startState();
    this.current_nurse = 8;
    this.current_status = 0;
    this.flatline = true;
  }

  void loopState()
  {
    super.loopState();
  }

  BearState changeState()
  {
    int change_time = 10;
    if (this.framesToSeconds(this.time) > change_time)
    {
      return new Start(this.game);
    } 
    else
    {
      return null;
    }
  }
}
